import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Set;


// README. This is an undirected, unweighted graph. In other words,
// edges do not have costs associated with them. If there is an edge 
// between nodes "A" and "B", then there is also an edge between "B"
// and "A". 

public class Graph<T> {
	public HashMap<T, Node> nodes;
	
	public Graph(){
		nodes = new HashMap<T, Node>();
	}
	
	public int size() {
		return nodes.size();
	}
	
	public Node getNode(T key){
		return nodes.get(key);
	}

	public boolean addNode(T e) {
		if (nodes.containsKey(e)) return false; // no duplicate nodes
		nodes.put(e, new Node(e));
		return true;
	}
	
	public boolean addEdge(T e1, T e2) {
		if (e1.equals(e2)) return false; // no circular edges on a node
		if (!nodes.containsKey(e1) || !nodes.containsKey(e2)) return false; // are the nodes actually in the graph
		ArrayList<Edge> ee = getNode(e1).neighbors;
		for (Edge e : ee) {
			if (e.otherNodeKey == e2) return false; // no duplicate edges
		}
		nodes.get(e1).addEdge(e2);
		ee = getNode(e2).neighbors;
		for (Edge e : ee) {
			if (e.otherNodeKey == e1) return false; // no duplicate edges
		}
		nodes.get(e2).addEdge(e1);
	    return true;
	}
	
	
	/**
	 * This method determines whether a graph is a binary graph. 
	 * By this we mean that every node has either zero, one or 
	 * two children. An empty graph is trivially a binary graph.
	 * 
	 * @return true if every node has either 0, 1, or 2 children
	 * and false otherwise
	 */
	public boolean isBinaryGraph() {
		//TODO: implement
	}
	

	/**
	 * This method adds the graph g to this graph. It will only 
	 * add additional nodes and edges that are not already 
	 * contained in this graph. 
	 * @param g Graph to be added
	 * @return This method returns true if all nodes and all edges 
	 * of g were added. It returns false otherwise.
	 */
	public boolean addGraph(Graph<T> g) {
		//TODO: Implement
	}
	
	/**
	 * This method returns a string of the node keys, in increasing 
	 * distance from the node passed to it. Distance is measured by 
	 * number of edges. We may call this a level-order traversal of 
	 * the graph. You may assume that the graph is connected. If two 
	 * nodes have the same distance, then preserve the ordering as
	 * found in the list of neighbors. Each node key should appear
	 * only once, in other words, please avoid cycles.
	 * @param start The node at which the traversal begins
	 * @return A list of the node keys, enclosed in square brackets 
	 * and separated by ", ". If the graph is empty, the method returns 
	 * just "[]".
	 */
	public String toString(T start) {
		//TODO: implement
	}

	
	public class Node {
		public T element;
		public ArrayList<Edge> neighbors;
		
		public Node(T e){
			element = e;
			neighbors = new ArrayList<Edge>();
		}
		
		public void addEdge(T e) {
			neighbors.add(new Edge(e));
		}
	}
	
	public class Edge {
		public T otherNodeKey;
		
		public Edge(T n){
			otherNodeKey = n;
		}
	}

}
