import static org.junit.Assert.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.junit.AfterClass;
import org.junit.Test;


public class Testing {
	private static int points = 0;
	
	@Test
	public void testIsBinaryGraph(){
		Graph<String> g = new Graph<>();
		assertTrue(g.isBinaryGraph());
		g.addNode("TerreHaute");
		assertTrue(g.isBinaryGraph());
		points += 1;
		g.addNode("Brazil");
		g.addNode("Sullivan");
		g.addNode("Clinton");
		assertTrue(g.isBinaryGraph());
		points += 1;
		g.addEdge("TerreHaute", "Sullivan");
		g.addEdge("TerreHaute", "Brazil");
		g.addEdge("Brazil", "Sullivan");
		assertTrue(g.isBinaryGraph());
		points += 1;
		g.addEdge("Brazil", "Clinton");
		assertFalse(g.isBinaryGraph());
		points += 1;	
		g.addEdge("TerreHaute", "Clinton");
		assertFalse(g.isBinaryGraph());
		points += 1;
		g.addEdge("Brazil", "Sullivan");
		assertFalse(g.isBinaryGraph());
		points += 1;
		
		g = new Graph<>();
		for (int i = 1; i < 1000000; i++) g.addNode(String.valueOf(i));
		assertTrue(g.isBinaryGraph());
		points += 1;
		for (int i = 1; i < (1000000-2); i++) {
			g.addEdge(String.valueOf(i), String.valueOf(i+1));
		}
		assertTrue(g.isBinaryGraph());
		points += 1;
		g.addEdge(String.valueOf(100), String.valueOf(102));
		assertFalse(g.isBinaryGraph());
		points += 2;
	}
	
	

	@Test
	public void testAddGraph(){
		Graph<String> g = new Graph<>();
		g.addNode("Berlin");
		g.addNode("Paris");
		g.addNode("London");
		g.addNode("Rome");	
		g.addEdge("Berlin", "Paris");
		g.addEdge("Paris", "London");
		assertFalse(g.addGraph(g));
		System.out.println(g.toString("Berlin"));
		// should be: [Berlin, Paris, London]
		points += 1;
		
		g = new Graph<>();
		g.addNode("Berlin");
		g.addNode("Paris");
		g.addNode("London");
		g.addNode("Rome");	
		g.addEdge("Berlin", "Paris");
		g.addEdge("Paris", "London");
		Graph<String> gg = new Graph<>();
		gg.addNode("London");
		gg.addNode("Rome");	
		gg.addEdge("London", "Rome");
		assertFalse(g.addGraph(gg));
		System.out.println(g.toString("Berlin"));
		// should be [Berlin, Paris, London, Rome]
		points += 4;

		g = new Graph<>();
		g.addNode("Berlin");
		g.addNode("Paris");
		g.addNode("London");
		g.addNode("Rome");	
		g.addEdge("Berlin", "Paris");
		g.addEdge("Paris", "London");
		g.addEdge("London", "Rome");
		gg = new Graph<>();
		gg.addNode("Monaco");
		gg.addNode("Madrid");	
		gg.addEdge("Monaco", "Madrid");
		assertFalse(g.addGraph(gg));
		System.out.println(g.toString("Monaco"));
		// should be [Monaco, Madrid]
		points += 5;
		
		gg = new Graph<>();
		gg.addNode("Paris");
		gg.addNode("Madrid");	
		gg.addEdge("Paris", "Madrid");
		assertFalse(g.addGraph(gg));
		System.out.println(g.toString("Monaco"));
		// should be [Monaco, Madrid, Paris, Berlin, London, Rome]
		points += 5;
		
		gg = new Graph<>();
		gg.addNode("Berlin");
		gg.addNode("London");	
		gg.addEdge("Berlin", "London");
		gg.addNode("Berlin");
		gg.addNode("Rome");	
		gg.addEdge("Berlin", "Rome");
		gg.addNode("Monaco");
		gg.addNode("London");	
		gg.addEdge("Monaco", "London");
		assertFalse(g.addGraph(gg));
		System.out.println(g.toString("Monaco"));
		// should be [Monaco, Madrid, London, Paris, Rome, Berlin]
		points += 5;

	}
	
	@Test
	public void testToString(){
		Graph<String> g = new Graph<>();
		assertEquals("[]", g.toString(null));
		points += 1;
		g.addNode("Munich");
		assertEquals("[]", g.toString("Berlin"));
		points += 1;
		assertEquals("[Munich]", g.toString("Munich"));
		points += 1;
		g.addNode("Hamburg");
		g.addNode("Berlin");
		g.addEdge("Hamburg", "Munich");
		g.addEdge("Munich", "Berlin");
		assertEquals("[Hamburg, Munich, Berlin]", g.toString("Hamburg"));
		points += 2;
		assertEquals("[Munich, Hamburg, Berlin]", g.toString("Munich"));
		points += 2;
		g.addNode("Paris");
		g.addNode("London");
		g.addNode("Madrid");
		g.addNode("Monaco");
		g.addEdge("Hamburg", "Paris");
		g.addEdge("Paris", "London");
		g.addEdge("London", "Monaco");
		g.addEdge("Monaco", "Madrid");
		assertEquals("[Hamburg, Munich, Paris, Berlin, London, Monaco, Madrid]", g.toString("Hamburg"));
		points += 2;
		g.addEdge("Berlin", "Madrid");
		assertEquals("[Hamburg, Munich, Paris, Berlin, London, Madrid, Monaco]", g.toString("Hamburg"));
		points += 3;
		
		g = new Graph<>();
		String testString = "[";
		String testString2 = "[";
		String testString3 = "[99, ";
		for (int i = 1; i < 100; i++) g.addNode(String.valueOf(i));
		for (int i = 1; i < (100-1); i++) {
			g.addEdge(String.valueOf(i), String.valueOf(i+1));
			testString += String.valueOf(i) + ", ";
			testString2 += String.valueOf(100-i) + ", ";
			if (i < 98) testString3 += String.valueOf(i) + ", ";
		}
		testString += "99]";
		testString2 += "1]";
		testString3 += "98]";
		assertEquals(testString, g.toString("1"));
		points += 3;
		assertEquals(testString2, g.toString("99"));
		points += 3;
		
		g = new Graph<>();
		for (int i = 1; i < 100; i++) g.addNode(String.valueOf(i));
		for (int i = 1; i < (100-1); i++) {
			for (int j = i+1; j < 100; j++) {
				g.addEdge(String.valueOf(i), String.valueOf(j));
			}
		}
		assertEquals(testString, g.toString("1"));
		points += 3;
		assertEquals(testString3, g.toString("99"));
		points += 3;
		
	}

	
	@AfterClass
	public static void testNothing(){
		System.out.println("Points: " + points);
	}
		

}
